/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Preferences = function()
{
	var preferences = {};
	
	var initialRememberDialogPositions;
	
	preferences.initPage = function()
	{
		// Save + Close buttons
		Buttons.addClickHandler('btnSave', Preferences.save);
		Buttons.addClickHandler('close', App.closeWindow);
		
		// Validation
		TabControl.addTabValidator('settings.appearance', preferences.validateAppearance);
		TabControl.addTabValidator('settings.plugins', preferences.validatePlugins);
		TabControl.addTabValidator('settings.mostRecent', preferences.validateMostRecent);
		
		// Font size + scale options
		Events.addEventHandler('fontsizeXS', 'click', Preferences.fontSizeOnClick);
		Events.addEventHandler('fontsizeS', 'click', Preferences.fontSizeOnClick);
		Events.addEventHandler('fontsizeM', 'click', Preferences.fontSizeOnClick);
		Events.addEventHandler('fontsizeL', 'click', Preferences.fontSizeOnClick);
		Events.addEventHandler('fontsizeXL', 'click', Preferences.fontSizeOnClick);
		Events.addEventHandler('scale', 'keypress', Events.integerOnly);

		// Color and date/time presets
		Buttons.addClickHandler('btnMinColor', Preferences.selectColorOnClick);
		Buttons.addClickHandler('btnMidColor', Preferences.selectColorOnClick);
		Buttons.addClickHandler('btnMaxColor', Preferences.selectColorOnClick);
		Events.addEventHandler('mincolor', 'change', Preferences.colorOnChange);
		Events.addEventHandler('midcolor', 'change', Preferences.colorOnChange);
		Events.addEventHandler('maxcolor', 'change', Preferences.colorOnChange);
		Events.addEventHandler('colorPresets', 'change', Preferences.colorPresetOnChange);
		Events.addEventHandler('datePresets', 'change', Preferences.datePresetOnChange);
		Events.addEventHandler('dateTimePresets', 'change', Preferences.dateTimePresetOnChange);
		
		// Recently used items
		Events.addEventHandler('maxViewed', 'keypress', Events.integerOnly);
		Events.addEventHandler('maxUsed', 'keypress', Events.integerOnly);
		Buttons.addClickHandler('moveUp', Preferences.moveStickyUpOrDown);
		Buttons.addClickHandler('moveDown', Preferences.moveStickyUpOrDown);
		Buttons.addClickHandler('moveLeft', Preferences.moveToSticky);
		Buttons.addClickHandler('moveRight', Preferences.moveFromSticky);
		Events.addEventHandler('stickyItems', 'dblclick', Preferences.moveFromSticky);
		Events.addEventHandler('allItems', 'dblclick', Preferences.moveToSticky);
		
		var frm = document.forms['preferences'];
		initialRememberDialogPositions = frm.remember_positions.checked;
		
		if (!App.localStorage())
		{
			frm.remember_positions.checked = false;
			frm.remember_positions.disabled = true;
			Doc.element('rememberPositions').title = 'Your browser doesn\'t support this feature';
			Doc.addClass('rememberPositions', 'disabled');
		}
	}

	// Validate the 'Appearance' tab
	preferences.validateAppearance = function()
	{
		var frm = document.forms['preferences'];
		var scale = parseInt(frm.scale.value);
		if (!Numbers.isInteger(frm.scale.value) || scale < 50 || scale > 200)
		{
			Forms.showNotification(frm.scale, 'Please enter a scale value between 50 and 200.');
			return false;
		}
		return true;
	}
	
	// Validate the 'Plugins' tab
	preferences.validatePlugins = function()
	{
		return true;
	}
	
	// Validate the 'Recent items' tab
	preferences.validateMostRecent = function()
	{
		var frm = document.forms['preferences'];
		var maxViewed = parseInt(frm.maxViewed.value);
		if (!Numbers.isInteger(frm.maxViewed.value) || maxViewed < 0 || maxViewed > 10)
		{
			Forms.showNotification(frm.maxViewed, 'Please enter a value between 0 and 10.');
			return false;
		}
		var maxUsed = parseInt(frm.maxUsed.value);
		if (!Numbers.isInteger(frm.maxUsed.value) || maxUsed < 0 || maxUsed > 10)
		{
			Forms.showNotification(frm.maxUsed, 'Please enter a value between 0 and 10.');
			return false;
		}
		return true;
	}

	// Save the preferences
	preferences.save = function()
	{
		var frm = document.forms['preferences'];
		if (TabControl.validateActiveTab('settings'))
		{
			var sticky_items = frm.sticky_items;
			for (var i = 0; i < sticky_items.length; i++)
			{
				sticky_items.options[i].selected = true;
			}
			
			if (initialRememberDialogPositions && !frm.remember_positions.checked)
			{
				Dialogs.forgetDialogPositions();
				Data.set(document.body, 'no-dialog-position', 1);
			}
			
			frm.submit();
		}
	}

	
	/**
		Open a popup that allows the user to select a color for 
		the given field.
	*/
	preferences.selectColorOnClick = function(event)
	{
		var dialogTitle = Data.get(event.currentTarget, 'dialog-title');
		var colorBox = Data.get(event.currentTarget, 'color-box');
		Dialogs.openColorChart(colorBox, dialogTitle);
	}
	
	/**
		Event handler that update the visual color box
		whenever the hidden rgb value is changed.
	 */
	preferences.colorOnChange = function(event)
	{
		var field = event.target.name;
		var display = Doc.element(field+'.visual');
		display.style.background = '#' + event.target.value;
	}
	
	/**
		Set a color for the given field.
	*/
	preferences.setColor = function(field, color)
	{
		var frm = document.forms['preferences'];
		frm[field].value = color;
		var display = Doc.element(field+'.visual');
		display.style.background = '#' + color;
	}
	
	/**
		Event handler that update all 3 colors from a given preset.
	*/
	preferences.colorPresetOnChange = function(event)
	{
		var colors = event.target.value.split(',');
		Preferences.setColor('mincolor', colors[0]);
		Preferences.setColor('midcolor', colors[1]);
		Preferences.setColor('maxcolor', colors[2]);
		event.target.selectedIndex = 0;
	}
	
	/**
		Event handler that update the date format from a given preset.
	 */
	preferences.datePresetOnChange = function(event)
	{
		var frm = document.forms['preferences'];
		frm.date_format.value = event.target.value;
		event.target.selectedIndex = 0;
	}

	/**
		Event handler that update the date+time format from a given preset.
	*/
	preferences.dateTimePresetOnChange = function(event)
	{
		var frm = document.forms['preferences'];
		frm.datetime_format.value = event.target.value;
		event.target.selectedIndex = 0;
	}

	/**
		Set a new value for the 'scale'
	*/
	preferences.fontSizeOnClick = function(event)
	{
		var frm = document.forms['preferences'];
		var newScale = Data.get(event.target, 'scale');
		frm.scale.value = newScale;
	}
	
	/**
		Move the selected options in the 'sticy items' list up or down.
	*/
	preferences.moveStickyUpOrDown = function(event)
	{
		var moveDown = Data.int(event.currentTarget, 'down');
		Forms.moveListOptions('stickyItems', moveDown);
	}

	preferences.moveToSticky = function()
	{
		var frm = document.forms['preferences'];
		Forms.moveSelected(frm.all_items, frm.sticky_items);
	}
	
	preferences.moveFromSticky = function()
	{
		var frm = document.forms['preferences'];
		Forms.moveSelected(frm.sticky_items, frm.all_items);
	}

	return preferences;
}();

Doc.onLoad(Preferences.initPage);


